require 'opal-browser'
require 'dom/debug'

# Add binding management to Browser::DOM::Node

class Browser
  class DOM
    class Document
      def active_element
        el = `#@native.activeElement()`
        if el
          Browser::DOM::Node.new(el)
        end
      end
    end
  end
end

class Browser
  class DOM
    class Element
    end
  end
end

class Browser
  class DOM
    class Node
      include CSR::DOM::Debug

      def as_native
        @native
      end

      # requires jquery and bootstrap.js - will fail otherwise
      def check_jquery
        %x(
          if (typeof jQuery === 'undefined') {
            throw new Error('Bootstrap\'s JavaScript requires jQuery');
          }
        )
      end

      def check_bootstrap
        %x(
          if (typeof($.fn.tooltip) === 'undefined') {
            throw new Error('Bootstrap.js not loaded');
          }
        )
      end

      # TODO: replicate this for all Bootstrap js features
      # TODO: refactor all bootstrap stuff to separate module
      # arg may be:
      # 0. nil - attaches tooltip to node
      # 1. hash of options and attaches tooltip to node
      # 2. 'show'
      # 3. 'hide'
      # 4. 'toggle'
      # 4. 'destroy'
      # see http://getbootstrap.com/javascript/#tooltips
      def tooltip(arg=nil)
        if String === arg
          arg = {
            title: arg,
            trigger: 'hover focus',
          }
        end
        debug 2, ->{[__FILE__, __LINE__, __method__, "arg=#{arg}"]}
        check_bootstrap
        debug 2, ->{[__FILE__, __LINE__, __method__, "@native=#{@native}"]}
        `$(#@native).tooltip(#{arg.to_n})`
        debug 2, ->{[__FILE__, __LINE__, __method__]}
      end

      def popover(arg=nil)
        debug 2, ->{[__FILE__, __LINE__, __method__]}
        if String === arg
          arg = {
            title: arg,
            trigger: 'hover focus',
          }
        end
        check_bootstrap
        `$(#@native).popover(#{arg.to_n})`
        debug 2, ->{[__FILE__, __LINE__, __method__]}
      end

      def bindings=(bindings)
        # store bindings in js element
        `#@native.voltBindings = #{bindings}`
      end

      def bindings
        # bindings stored in js element
        b = `#@native.voltBindings`
        `b === undefined` ? nil : b
      end

      # bindings set (if required) by Paggio::HTML::Element
      def start_bindings
        if bindings
          bindings.each do |b|
            # cannot use self as self is transient
            b.start(self.id)
          end
        end
      end

      def stop_bindings
        if bindings
          bindings.each do |b|
            b.stop
          end
        end
      end

      def bindings_count
        b = bindings
        b ? b.size : 0
      end

    end
  end
end