module CSR
  module DOM
    module Debug
      module_function

      def stack_trace
        callback = ->(stackframes) {
          %x(
            var stringifiedStack = stackframes.map(function(sf) {
              return sf.toString();
            }).join('\n');
            console.log(stringifiedStack);
          )
        }
        errback = ->(err) {
          `console.log(err.message)`
        }
        `StackTrace.get().then(callback).catch(errback)`
      end

      def at_debug_level(l)
        prev= debug_level
        self.debug_level = 1
        result = yield
        self.debug_level = prev
        result
      end

      def debug_level=(l)
        @debug_level = l
      end

      def debug_level
        @debug_level ||= 0
      end

      def debug_method_missing=(v)
        @debug_method_missing = v
      end

      def debug_method_missing?
        !!@debug_method_missing
      end

      def method_missing(name, *args, &block)
        if true # debug_method_missing?
          debug 0, ->{[__FILE__, __LINE__, __method__, "METHOD_MISSING: self=#{self} method=#{name}"]}
          # debug 0, ->{[__FILE__, __LINE__, __method__, "called by: #{caller.first.to_s}"]}
          stack_trace
        end
        super
      end

      def debug(level, proc)
        if level == 0 || level <= debug_level
          file, line, method, msg = proc.call
          s = "#{file}[#{line}] #{self.is_a?(Class) ? (self.name + '#') : self.class.name}##{method}"
          s = s + " >> #{msg}" if msg
          if RUBY_PLATFORM == 'opal'
            `console.log(s)`
          else
            puts s
          end
        end
      end

    end
  end
end
