require 'dom/content'
require 'dom/builder'
require 'dom/util'
require 'dom/debug'

module CSR
  module DOM
    class Table
      class Caption
        include CSR::DOM::Builder
        include CSR::DOM::Debug

        attr_reader :table, :content

        def initialize(table, content)
          # self.debug_level = 1
          @table = table
          @content = content
        end

        def update
          if @root
            debug 1, ->{[ __FILE__, __LINE__,  __method__ ]}
            animate do
              @root.clear
              @root << resolve_content
            end
          end
        end

        def root
          debug 1, ->{[ __FILE__, __LINE__,  __method__ ]}
          @root ||= tag(
            :caption,
            attributes: @table.caption_attributes,
            content: resolve_content
          )
          debug 1, ->{[ __FILE__, __LINE__,  __method__, "@root=#{@root}" ]}
          @root
        end

        def resolve_content
          result = case content
            when Content
              content.element(:caption, attributes: attributes, context: @table)
            when Proc
              content.call
            else
              content
          end
          # debug 1, ->{[ __FILE__, __LINE__,  __method__, "result=#{result}" ]}
          result
        end
      end


    end
  end
end