require 'dom/content'

module CSR
  module DOM
    class Table
      class Column

        attr_reader :id             # unique in table, default to field

        # Content procs with arity 2 should expect context to be [table, section, row]
        # where table is a Table; section is :head, :body or :foot; row is an element in
        # head_rows, body_rows or foot_rows.

        attr_reader :head   # Content or String or nil.
        attr_reader :body   # Content or String or nil.
        attr_reader :foot   # Content or String or nil.

        # OPTIONS:
        # :id     # unique in table
        # :sort   # column may be sorted : boolean or nil
        # :hide   # column may be hidden : boolean or nil
        # :master # column used for grouping in accordion table
        # :head   # Content or String or nil for column head
        # :body   # Content or String or nil for column body
        # :foot   # Content or String or nil for column foot

        # TODO:
        # width:

        def initialize(**options)
          @id = options[:id]
          @sort_callback = options[:sort]
          @hide = !!options[:hide]
          @master = !!options[:master]
          @head = options[:head]
          @body = options[:body]
          @foot = options[:foot]
        end

        def hash
          @id.hash
        end

        def eql?(other)
          @id == other.id
        end

        def ==(other)
          @id == other.id
        end

        def master?
          @master
        end

        def sort_callback
          @sort_callback
        end

        def sort?
          !!@sort_callback
        end

        def hide?
          @hide
        end

        # section is :head, :body or :foot
        def section_content(section)
          send(section.to_sym)
        end

        # Returns a (probably) unique column id.
        def self.unique_id
          @@id_chars ||= ('0'..'9').to_a + ('A'..'F').to_a
          result = 'COL_'
          8.times { result = result + @@id_chars.sample }
          result
        end

      end
    end
end
end
