require 'dom/table/column'
require 'dom/builder'
require 'dom/util'
require 'dom/debug'

module CSR
  module DOM
    class Table
      class Row
        include CSR::DOM::Builder
        include CSR::DOM::Util
        include CSR::DOM::Debug

        attr_reader :table, :section, :index

        def initialize(table, section, index)
          # self.debug_level = 1
          @table, @section, @index = table, section, index
        end

        def root
          debug 1, ->{[ __FILE__, __LINE__, __method__, "@root=#{@root}" ]}
          @root ||= tag(
            :tr,
            attributes: attributes,
            content: cells
          )
          debug 1, ->{[ __FILE__, __LINE__,  __method__, "@root=#{@root}" ]}
          @root
        end

        def attributes
          debug 1, ->{[ __FILE__, __LINE__,  __method__, "@section_id=#{@section_id} @index=@{index}" ]}
          result = section.row_attributes(@index)
          debug 1, ->{[ __FILE__, __LINE__,  __method__, "result=#{result}" ]}
          result
        end

        # columns may be indexes in table columns or Column's
        def update(_columns = nil)
          debug 1, ->{[ __FILE__, __LINE__,  __method__, "_columns=#{_columns}" ]}
          if @root
            animate do
              if _columns
                columns = _columns.map { |c| Integer === c ? @table.columns[c] : c }
                debug 1, ->{[ __FILE__, __LINE__,  __method__, "columns=#{columns.map{|e|e.id}}" ]}
                @table.visible_columns.each_with_index do |c, i|
                  if columns.include?(c)
                    debug 1, ->{[ __FILE__, __LINE__,  __method__, "updating column[#{i}] #{c.id}" ]}
                    new_cell = cell(c)
                    @cells[i].replace(new_cell)
                    @cells[i] = new_cell
                  end
                end
              else
                debug 1, ->{[ __FILE__, __LINE__,  __method__, "updating all columns" ]}
                @cells = nil
                @root.clear
                @root << cells
              end
            end
          end
        end

        # Returns array of cell elements in table's visible column order
        def cells
          unless @cells
            column_index = -1
            @cells = @table.visible_columns.map do |column|
              column_index += 1
              cell(column, column_index)
            end
          end
          @cells
        end

        def head?
          @section.head?
        end
        
        def row_source
          @section.row_source(@index)
        end

        def cell(column, column_index)
          attributes, content = cell_attributes_and_content(column, column_index)
          tag(
            head? ? :th : :td,
            attributes: attributes,
            content: content
          )
        end

        def cell_attributes_and_content(column, column_index)
          content = column.section_content(@section.id)
          attributes = nil
          debug 1, ->{[ __FILE__, __LINE__,  __method__, "section.id=#{@section.id} row_index=#{@index} column.id=#{column.id} content=#{content}" ]}
          if Content === content
            content, attributes = content.value_and_attributes(context: row_source)
            debug 1, ->{[ __FILE__, __LINE__,  __method__, "section.id=#{@section.id} row_index=#{@index} column.id=#{column.id} content=#{content} attributes=#{attributes}" ]}
          end
          if head? && column.sort?
            debug 1, ->{[ __FILE__, __LINE__,  __method__, "section.id=#{@section.id} row_index=#{@index} column.id=#{column.id} content=#{content}" ]}
            content = div_with_sort_icon(
              callback: column.sort_callback,
              direction: @table.sort_column_id == column.id ? @table.sort_order : 0,
              content: content
            )
            debug 1, ->{[ __FILE__, __LINE__,  __method__, "section.id=#{@section.id} column.id=#{column.id} content=#{content}" ]}
          end
          attributes ||= {}
          attributes[:id] ||= hex_id
          [attributes, content]
        end

      end

    end
  end
end