require 'opal-browser'
require 'browser/animation_frame'

module CSR
  module DOM
    module Util
      module_function

      def set_interval(milliseconds, callback)
        `setInterval(callback, milliseconds)`
      end

      def clear_interval(interval)
        `clearInterval(interval)` if interval
      end

      def cursor(which, &block)
        current = $document.body.style.cursor
        $document.body.style.cursor = which
        if block
          yield
          $document.body.style.cursor = current
        end
      end

      def cursor_wait(&block)
        cursor('wait', &block)
      end

      def cursor_auto(&block)
        cursor('auto', &block)
      end

      def cursor_normal(&block)
        cursor_auto(&block)
      end

      def animate(&block)
        # Browser::AnimationFrame.new(window, &block)
        animation_frame(&block)
      end

      # Returns a (probably) unique element id
      # composed of hexadecimal digits.
      def hex_id
        @@hex_digits ||= ('0'..'9').to_a + ('A'..'F').to_a
        result = ''
        8.times { result = result + @@hex_digits.sample }
        result
      end

      # args are hashes, lowest to highest precedence,
      # i.e. later args override earlier args
      # goes deep recursively
      def merge_attributes(*args)
        result = args[0] || {}
        args.each do |arg|
          if arg
            arg.each_pair do |key, next_value|
              current_value = result[key]
              if Hash === current_value && Hash === next_value
                result[key] = merge_attributes(current_value, next_value)
              else
                result[key] = next_value
              end
            end
          end
        end
        result
      end

      # make a single array from args
      # args may be mix of non-enumerables and enumerables
      # nils will not be included
      def arrify(*args)
        result = []
        args.each do |arg|
          if arg
            if Enumerable === arg
              arg.each do |e|
                result << e if e
              end
            else
              result << arg if arg
            end
          end
        end
        result
      end

      def normalize_style(style)
        unless style.respond_to?(:to_h)
          raise TypeError, "#{__FILE__}[#{__LINE__}] : style #{style.class} must respond to :to_h"
        end
        # debug 1, ->{[ __FILE__, __LINE__, __method__, "style=#{style}" ]}
        result = {}
        style.to_h.each do |k,v|
          k = k.to_s.gsub('_', '-')
          result[k] = v.to_s
        end
        # debug 1, ->{[ __FILE__, __LINE__, __method__, "result=#{result}" ]}
        result
      end
    end
  end
end
